/* -*- Mode: C; c-basic-offset: 2; indent-tabs-mode: nil -*-
 *
 * Pigment media rendering library
 *
 * Copyright © 2006, 2007 Fluendo Embedded S.L.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author(s): Loïc Molinari <loic@fluendo.com>
 *            Julien Moutte <julien@fluendo.com>
 */

#ifndef __PGM_DRAWABLE_H__
#define __PGM_DRAWABLE_H__

#include <gst/gst.h>
#include "pgmcommon.h"

G_BEGIN_DECLS

#define PGM_TYPE_DRAWABLE \
  (pgm_drawable_get_type ())
#define PGM_DRAWABLE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj), PGM_TYPE_DRAWABLE, PgmDrawable))
#define PGM_DRAWABLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass), PGM_TYPE_DRAWABLE, PgmDrawableClass))
#define PGM_IS_DRAWABLE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj), PGM_TYPE_DRAWABLE))
#define PGM_IS_DRAWABLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE((klass), PGM_TYPE_DRAWABLE))
#define PGM_DRAWABLE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS((obj), PGM_TYPE_DRAWABLE, PgmDrawableClass))

typedef struct _PgmDrawable      PgmDrawable;
typedef struct _PgmDrawableClass PgmDrawableClass;

/**
 * PgmDrawableLayer:
 * @PGM_DRAWABLE_UNBINDED: Not binded to a #Canvas.
 * @PGM_DRAWABLE_FAR: Layer of drawables drawn behind the other layers.
 * @PGM_DRAWABLE_MIDDLE: Layer of drawables drawn over the drawables in the
 * far layer and behind the drawables in the near one.
 * @PGM_DRAWABLE_NEAR: Layer of drawables drawn over the other layers.
 *
 * The rendering layers.
 */
typedef enum {
  PGM_DRAWABLE_UNBINDED,
  PGM_DRAWABLE_FAR,
  PGM_DRAWABLE_MIDDLE,
  PGM_DRAWABLE_NEAR
} PgmDrawableLayer;

/**
 * PgmDrawableProperty:
 * @PGM_DRAWABLE_VISIBILITY: %PgmDrawable visibility property.
 * @PGM_DRAWABLE_SIZE: %PgmDrawable size property.
 * @PGM_DRAWABLE_POSITION: %PgmDrawable position property.
 * @PGM_DRAWABLE_BG_COLOR: %PgmDrawable background color property.
 * @PGM_DRAWABLE_FG_COLOR: %PgmDrawable foreground color property.
 * @PGM_DRAWABLE_OPACITY: %PgmDrawable opacity property.
 * @PGM_DRAWABLE_ARC: %PgmDrawable arc property.
 * @PGM_DRAWABLE_STROKE_WIDTH: %PgmDrawable stroke width property.
 * @PGM_DRAWABLE_STROKE_COLOR: %PgmDrawable stroke color property.
 * @PGM_DRAWABLE_SHADOW_RADIUS: %PgmDrawable shadow radius property.
 * @PGM_DRAWABLE_SHADOW_OFFSET: %PgmDrawable shadow offset property.
 * @PGM_DRAWABLE_SHADOW_COLOR: %PgmDrawable shadow color property.
 * @PGM_IMAGE_DATA_BUFFER: %PgmImage buffer data property.
 * @PGM_IMAGE_DATA_GST_BUFFER: %PgmImage GstBuffer data property.
 * @PGM_IMAGE_DATA_FD: %PgmImage file descriptor data property.
 * @PGM_IMAGE_DATA_IMAGE: %PgmImage image data property.
 * @PGM_IMAGE_DATA_EMPTY: %PgmImage empty data property.
 * @PGM_IMAGE_ALIGNMENT: %PgmImage alignment property.
 * @PGM_IMAGE_LAYOUT: %PgmImage layout property.
 * @PGM_IMAGE_INTERP: %PgmImage interpolation property.
 * @PGM_IMAGE_ASPECT_RATIO: %PgmImage aspect-ratio property.
 * @PGM_TEXT_LABEL: %PgmText label property.
 * @PGM_TEXT_MARKUP: %PgmText markup property.
 * @PGM_TEXT_FONT_FAMILY: %PgmText font family property.
 * @PGM_TEXT_HEIGHT: %PgmText height property.
 * @PGM_TEXT_ELLIPSIZE: %PgmText ellipsize property.
 * @PGM_TEXT_JUSTIFY: %PgmText justify property.
 * @PGM_TEXT_ALIGNMENT: %PgmText alignment property.
 * @PGM_TEXT_WRAP: %PgmText wrap property.
 * @PGM_TEXT_GRAVITY: %PgmText gravity property.
 * @PGM_TEXT_STRETCH: %PgmText stretch property.
 * @PGM_TEXT_STYLE: %PgmText style property.
 * @PGM_TEXT_VARIANT: %PgmText variant property.
 * @PGM_TEXT_WEIGHT: %PgmText weight property.
 * @PGM_TEXT_LINE_SPACING: %PgmText line spacing property.
 * @PGM_TEXT_OUTLINE_COLOR: %PgmText outline color property.
 * @PGM_TEXT_OUTLINE_WIDTH: %PgmText outline width property.
 *
 * The different properties of the drawables.
 */
typedef enum {
  /* PgmDrawable properties */
  PGM_DRAWABLE_VISIBILITY,
  PGM_DRAWABLE_SIZE,
  PGM_DRAWABLE_POSITION,
  PGM_DRAWABLE_BG_COLOR,
  PGM_DRAWABLE_FG_COLOR,
  PGM_DRAWABLE_OPACITY,
  PGM_DRAWABLE_ARC,
  PGM_DRAWABLE_STROKE_WIDTH,
  PGM_DRAWABLE_STROKE_COLOR,
  PGM_DRAWABLE_SHADOW_RADIUS,
  PGM_DRAWABLE_SHADOW_OFFSET,
  PGM_DRAWABLE_SHADOW_COLOR,
  /* PgmImage properties */
  PGM_IMAGE_DATA_BUFFER,
  PGM_IMAGE_DATA_GST_BUFFER,
  PGM_IMAGE_DATA_FD,
  PGM_IMAGE_DATA_IMAGE,
  PGM_IMAGE_DATA_EMPTY,
  PGM_IMAGE_ALIGNMENT,
  PGM_IMAGE_LAYOUT,
  PGM_IMAGE_INTERP,
  PGM_IMAGE_ASPECT_RATIO,
  /* PgmText properties */
  PGM_TEXT_LABEL,
  PGM_TEXT_MARKUP,
  PGM_TEXT_FONT_FAMILY,
  PGM_TEXT_HEIGHT,
  PGM_TEXT_ELLIPSIZE,
  PGM_TEXT_JUSTIFY,
  PGM_TEXT_ALIGNMENT,
  PGM_TEXT_WRAP,
  PGM_TEXT_GRAVITY,
  PGM_TEXT_STRETCH,
  PGM_TEXT_STYLE,
  PGM_TEXT_VARIANT,
  PGM_TEXT_WEIGHT,
  PGM_TEXT_LINE_SPACING,
  PGM_TEXT_OUTLINE_COLOR,
  PGM_TEXT_OUTLINE_WIDTH
} PgmDrawableProperty;

/**
 * PgmDrawableFlags:
 * @PGM_DRAWABLE_FLAG_UNPARENTING: Child is being removed from the parent
 * canvas. pgm_canvas_del_drawable() on a child already being removed
 * immediately returns %FALSE.
 * @PGM_DRAWABLE_FLAG_VISIBLE: Drawable is visible.
 * @PGM_DRAWABLE_FLAG_LAST: Offset to define more flags.
 *
 * The standard flags that a drawable may have.
 */
typedef enum {
  PGM_DRAWABLE_FLAG_UNPARENTING = (GST_OBJECT_FLAG_LAST << 0),
  PGM_DRAWABLE_FLAG_VISIBLE     = (GST_OBJECT_FLAG_LAST << 1),
  /* padding */
  PGM_DRAWABLE_FLAG_LAST        = (GST_OBJECT_FLAG_LAST << 16)
} PgmDrawableFlags;

/* Drawable signals */
typedef enum {
  PGM_DRAWABLE_CHANGED,
  PGM_DRAWABLE_LAST_SIGNAL
} PgmDrawableSignals;

/**
 * PgmDrawable:
 * @layer: the layer the drawable belongs to.
 * @x: the drawable position on the x axis.
 * @y: the drawable position on the y axis.
 * @z: the drawable position on the z axis.
 * @width: the drawable width.
 * @height: the drawable height.
 * @fg_r: the drawable foreground red color component.
 * @fg_g: the drawable foreground green color component.
 * @fg_b: the drawable foreground blue color component.
 * @fg_a: the drawable foreground alpha color component.
 * @bg_r: the drawable background red color component.
 * @bg_g: the drawable background green color component.
 * @bg_b: the drawable background blue color component.
 * @bg_a: the drawable background alpha color component.
 * @opacity: the drawable opacity.
 * @arc_width: the drawable arc width.
 * @arc_height: the drawable arc height.
 * @stroke_width: the drawable stroke width.
 * @stroke_r: the drawable stroke red color component.
 * @stroke_g: the drawable stroke green color component.
 * @stroke_b: the drawable stroke blue color component.
 * @stroke_a: the drawable stroke alpha color component.
 * @shadow_radius: the drawable shadow radius.
 * @shadow_offset_x: the drawable shadow x offset.
 * @shadow_offset_y: the drawable shadow y offset.
 * @shadow_r: the drawable shadow red color component.
 * @shadow_g: the drawable shadow green color component.
 * @shadow_b: the drawable shadow blue color component.
 * @shadow_a: the drawable shadow alpha color component.
 *
 * Pigment drawable abstract base class.
 */
struct _PgmDrawable {
  GstObject parent;

  /*< public >*/
  PgmDrawableLayer layer;

  /* Position */
  gfloat x, y, z;

  /* Size */
  gfloat width, height;

  /* Colors */
  guchar fg_r, fg_g, fg_b, fg_a;
  guchar bg_r, bg_g, bg_b, bg_a;

  /* Opacity */
  guchar opacity;

  /* Arc */
  gfloat arc_width, arc_height;

  /* Stroke */
  gfloat stroke_width;
  guchar stroke_r, stroke_g, stroke_b, stroke_a;

  /* Shadow */
  gfloat shadow_radius;
  gfloat shadow_offset_x, shadow_offset_y;
  guchar shadow_r, shadow_g, shadow_b, shadow_a;

  /*< private >*/
  /* Padding */
  gpointer _pgm_reserved[PGM_PADDING];
};

/**
 * PgmDrawableClass:
 * @parent_class: the parent class structure.
 * @changed: the "changed" signal.
 * @hide: the hide virtual method.
 * @show: the show virtual method.
 * @set_size: the set_size virtual method.
 * @set_position: the set_position virtual method.
 * @set_fg_color: the set_fg_color virtual method.
 * @set_bg_color: the set_bg_color virtual method.
 * @set_opacity: the set_opacity virtual method.
 * @set_arc: the set_arc virtual method.
 * @set_stroke_width: the set_stroke_width virtual method.
 * @set_stroke_color: the set_stroke_color virtual method.
 * @set_shadow_radius: the set_shadow_radius virtual method.
 * @set_shadow_offset: the set_shadow_offset virtual method.
 * @set_shadow_color: the set_shadow_color virtual method.
 *
 * The #PgmDrawable structure.
 */
struct _PgmDrawableClass {
  GstObjectClass parent_class;

  /*< public >*/

  /* Signals */
  void     (*changed)           (PgmDrawable *drawable,
                                 PgmDrawableProperty property);

  /* virtual methods for subclasses */
  PgmError (*hide)              (PgmDrawable *drawable);
  PgmError (*show)              (PgmDrawable *drawable);

  PgmError (*set_size)          (PgmDrawable *drawable,
                                 gfloat width,
                                 gfloat height);

  PgmError (*set_position)      (PgmDrawable *drawable,
                                 gfloat x,
                                 gfloat y,
                                 gfloat z);

  PgmError (*set_fg_color)      (PgmDrawable *drawable,
                                 guchar r,
                                 guchar g,
                                 guchar b,
                                 guchar a);

  PgmError (*set_bg_color)      (PgmDrawable *drawable,
                                 guchar r,
                                 guchar g,
                                 guchar b,
                                 guchar a);

  PgmError (*set_opacity)       (PgmDrawable *drawable,
                                 guchar opacity);

  PgmError (*set_arc)           (PgmDrawable *drawable,
                                 gfloat width,
                                 gfloat height);

  PgmError (*set_stroke_width)  (PgmDrawable *drawable,
                                 gfloat width);

  PgmError (*set_stroke_color)  (PgmDrawable *drawable,
                                 guchar r,
                                 guchar g,
                                 guchar b,
                                 guchar a);

  PgmError (*set_shadow_radius) (PgmDrawable *drawable,
                                 gfloat radius);

  PgmError (*set_shadow_offset) (PgmDrawable *drawable,
                                 gfloat x,
                                 gfloat y);

  PgmError (*set_shadow_color)  (PgmDrawable *drawable,
                                 guchar r,
                                 guchar g,
                                 guchar b,
                                 guchar a);

  /*< private >*/
  gpointer _pgm_reserved[PGM_PADDING];
};

/* Allows emission of signals from sub-classes */
extern guint pgm_drawable_signals[PGM_DRAWABLE_LAST_SIGNAL];

GType    pgm_drawable_get_type        (void);

PgmError pgm_drawable_hide            (PgmDrawable *drawable);
PgmError pgm_drawable_show            (PgmDrawable *drawable);
PgmError pgm_drawable_is_visible      (PgmDrawable *drawable,
                                       gboolean *visible);

PgmError pgm_drawable_set_size        (PgmDrawable *drawable,
                                       gfloat width,
                                       gfloat height);
PgmError pgm_drawable_get_size        (PgmDrawable *drawable,
                                       gfloat *width,
                                       gfloat *height);

PgmError pgm_drawable_set_position    (PgmDrawable *drawable,
                                       gfloat x,
                                       gfloat y,
                                       gfloat z);
PgmError pgm_drawable_get_position    (PgmDrawable *drawable,
                                       gfloat *x,
                                       gfloat *y,
                                       gfloat *z);

PgmError pgm_drawable_set_fg_color    (PgmDrawable *drawable,
                                       guchar r,
                                       guchar g,
                                       guchar b,
                                       guchar a);
PgmError pgm_drawable_get_fg_color    (PgmDrawable *drawable,
                                       guchar *r,
                                       guchar *g,
                                       guchar *b,
                                       guchar *a);

PgmError pgm_drawable_set_bg_color    (PgmDrawable *drawable,
                                       guchar r,
                                       guchar g,
                                       guchar b,
                                       guchar a);
PgmError pgm_drawable_get_bg_color    (PgmDrawable *drawable,
                                       guchar *r,
                                       guchar *g,
                                       guchar *b,
                                       guchar *a);

PgmError pgm_drawable_set_opacity     (PgmDrawable *drawable,
                                       guchar opacity);
PgmError pgm_drawable_get_opacity     (PgmDrawable *drawable,
                                       guchar *opacity);

PgmError pgm_drawable_set_arc         (PgmDrawable *drawable,
                                       gfloat width,
                                       gfloat height);
PgmError pgm_drawable_get_arc         (PgmDrawable *drawable,
                                       gfloat *width,
                                       gfloat *height);

/* Stroke methods */

PgmError pgm_drawable_set_stroke_width (PgmDrawable *drawable,
                                        gfloat width);
PgmError pgm_drawable_get_stroke_width (PgmDrawable *drawable,
                                        gfloat *width);

PgmError pgm_drawable_set_stroke_color (PgmDrawable *drawable,
                                        guchar r,
                                        guchar g,
                                        guchar b,
                                        guchar a);
PgmError pgm_drawable_get_stroke_color (PgmDrawable *drawable,
                                        guchar *r,
                                        guchar *g,
                                        guchar *b,
                                        guchar *a);

/* Shadow methods */

PgmError pgm_drawable_set_shadow_radius (PgmDrawable *drawable,
                                         gfloat radius);
PgmError pgm_drawable_get_shadow_radius (PgmDrawable *drawable,
                                         gfloat *radius);

PgmError pgm_drawable_set_shadow_offset (PgmDrawable *drawable,
                                         gfloat x,
                                         gfloat y);
PgmError pgm_drawable_get_shadow_offset (PgmDrawable *drawable,
                                         gfloat *x,
                                         gfloat *y);

PgmError pgm_drawable_set_shadow_color  (PgmDrawable *drawable,
                                         guchar r,
                                         guchar g,
                                         guchar b,
                                         guchar a);
PgmError pgm_drawable_get_shadow_color  (PgmDrawable *drawable,
                                         guchar *r,
                                         guchar *g,
                                         guchar *b,
                                         guchar *a);

G_END_DECLS

#endif /* __PGM_DRAWABLE_H__ */
